#include "Pokitto.h"
#include "bitmaps.h"
#include "sounds.h"
#include "HWSound.h"

Pokitto::Core game;

// screen width and height for bounds checking on drawPixel
#include "drawing.h"

#define _8BPP 8
#define _4BPP 4
#define _2BPP 2
#define gui_black  3
#define gui_dkgrey  2
#define gui_ltgrey  1
#define gui_white  0
#define max_windows 20
//------------------------[ JoyHat ]------------------------------------------------
mbed::DigitalOut jh_rumble(EXT0);
mbed::DigitalIn jh_b1(EXT15);
mbed::DigitalIn jh_b2(EXT14);
mbed::AnalogIn jh_x(EXT1);
mbed::AnalogIn jh_y(EXT2);

//------------------------[ Button handling, very accurate ]------------------------
#define HELD 0
#define NEW 1
#define RELEASE 2
byte CompletePad, ExPad, TempPad, myPad;
bool _A[3], _B[3], _C[3], _Up[3], _Down[3], _Left[3], _Right[3];

DigitalIn _aPin(P1_9);
DigitalIn _bPin(P1_4);
DigitalIn _cPin(P1_10);
DigitalIn _upPin(P1_13);
DigitalIn _downPin(P1_3);
DigitalIn _leftPin(P1_25);
DigitalIn _rightPin(P1_7);

void UPDATEPAD(int pad, int var) {
  _C[pad] = (var >> 1)&1;
  _B[pad] = (var >> 2)&1;
  _A[pad] = (var >> 3)&1;
  _Down[pad] = (var >> 4)&1;
  _Left[pad] = (var >> 5)&1;
  _Right[pad] = (var >> 6)&1;
  _Up[pad] = (var >> 7)&1;
}

void UpdatePad(int joy_code){
  ExPad = CompletePad;
  CompletePad = joy_code;
  UPDATEPAD(HELD, CompletePad); // held
  UPDATEPAD(RELEASE, (ExPad & (~CompletePad))); // released
  UPDATEPAD(NEW, (CompletePad & (~ExPad))); // newpress
}

byte updateButtons(byte var){
   var = 0;
   if (_cPin) var |= (1<<1);
   if (_bPin) var |= (1<<2);
   if (_aPin) var |= (1<<3); // P1_9 = A
   if (_downPin) var |= (1<<4);
   if (_leftPin) var |= (1<<5);
   if (_rightPin) var |= (1<<6);
   if (_upPin) var |= (1<<7);

   return var;
}
//----------------------------------------------------------------------------------

//Sprite structure
typedef struct{
    int x,y,jcx,jcy;  //location
    int xSpeed, ySpeed; //speed
    int analog = 511;
    float decay = .55;
}Mouse;
Mouse myMouse;

typedef struct{
    int x,y,w,h;
    char title[24]; // "Window Settings        " // max length
    int function; // what the windows does
    bool inUse=0; // is window being used?
    int zOrder=0;
    bool drag=false;
    int dragX,dragY;
}Win;
Win window[max_windows]; // max 10 windows

time_t pokTime = time(NULL);
const char *wDays[]={"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
const char *wDaysFull[]={"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"};
const char *mDays[]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Nov","Dec"};
const char *mDaysFull[]={"January","Febuary","March","April","May","June","July","August","September","November","December"};
int _backdrop=0;
int backdropY; // store backdrop windows top for other windows offest

typedef struct{
	int x, y; // x/y position
} playertype;

void clearScreen(){
    // clear the screen
    for(uint16_t t=0; t<110*88; t++){
        game.display.screenbuffer[t]=0;
    }
}


/****************************************************************************/
char tempText[255];
// print text
void print(int16_t x, uint16_t y, const char* text, int justify=0){
    int numChars = strlen(text);

    // justify
    // 0 = left - default
    // 1 = center
    // 2 = right

    if(justify == 1){
        int num=0;
        for(uint8_t t = 0; t < numChars; t++){
            num+=4;//letter_widths[text[t]-32];
        }
        x = x - (num/2);
    }

    if(justify == 2){
        int num=0;
        for(uint8_t t = 0; t < numChars; t++){
            num+=4;//letter_widths[text[t]-32];
        }
        x = x - num;
    }


    // 0 = 8bit
    // 1 = 4bit
    // 2 = 2bit
//    for(int t=y/2; t<(y/2)+3; t++){ // set bitDepth for text lines
//        game.display.scanType[t]=2; // 2bit
//    }

    for(uint8_t t = 0; t < numChars; t++){
        uint8_t character = text[t]-32;
        drawBitmap(x,y,tinyfont[character], 2);
        x += 4;//letter_widths[character-32];
    }
}






/****************************************************************************/

int toTime = 0;
int curTime = 0;



time_t asUnixTime(int year, int mon, int mday, int hour, int min, int sec) {
    struct tm   t;
    t.tm_year = year - 1900;
    t.tm_mon =  mon;
    t.tm_mday = mday;
    t.tm_hour = hour;
    t.tm_min = min;
    t.tm_sec = sec;
    t.tm_isdst = -1;            // Is Daylight saving time on? 1 = yes, 0 = no, -1 = unknown

    return mktime(&t);          // returns seconds elapsed since January 1, 1970 (begin of the Epoch)
}

void setMyTime(int day, int month, int year, int hour, int minute, int second=0){
/*
    // 1st Jan 2019
    int year = 2019;    //  year
    int month = 3;      //  0-11 (jan = 0)
    int day = 15;       //  1-31 day of month
    int hour = 13;       //  0-32 hours from midnight
    int minute = 42;     //  0-59 minutes this hour
    int second = 0;     //  0-59 seconds this minute
*/
    toTime = asUnixTime(year, month, day, hour, minute, second);
    set_time(toTime);  // 2000, jan, 1st, 00:00:00
}


void startup(){
    game.begin();
    game.display.persistence=1;
    game.setFrameRate(255); // max out the frame rate so that it doesn't get used
    game.display.setInvisibleColor(-1);

    load8bitPalette(seein_angel_pal,0,255);
    load4bitPalette(leaves_pal,0,15);
    load2bitPalette(tinyfont_pal,0,3);

    clearScreen();

    pokPlayStream();

    // set Draw mode to hires 4 colour
    for(int t=0; t<88; t++){
        game.display.scanType[t]=2; // 2bit
    }

    myMouse.jcx = ((jh_x.read()*myMouse.analog));
    myMouse.jcy = ((jh_y.read()*myMouse.analog));

}

void drawFillRect2(int x, int y, int w, int h, int col){

    // limit to screen
    if(x<0)x=0;
    if(w>220)w=220;
    if(y<0)y=0;
    if(h>176)h=176;

    int xStart = (x/4)+1;
    int xEnd = (w/4);

    int byteColor = (col<<6)|(col<<4)|(col<<2)|col;
    uint8_t column = x&3;

    for(int y1 = y; y1< h; y1++){

        // left edge
        int i = y1*55 + (x>>2);
        uint16_t pixel = game.display.screenbuffer[i];
        col &= 3; // just in case
        uint8_t column = x&3;
             if (column==3) pixel = (pixel & B11111100) | (byteColor & B00000011);
        else if (column==2) pixel = (pixel & B11110000) | (byteColor & B00001111);
        else if (column==1) pixel = (pixel & B11000000) | (byteColor & B00111111);
        else if (column==0) pixel = (pixel & B00000000) | (byteColor & B11111111);
        game.display.screenbuffer[i] = pixel;

        // middle
        for(int x1 = xStart; x1< xEnd; x1++){
            game.display.screenbuffer[x1+55*y1] = byteColor;
        }

        // right edge
        i = y1*55 + (w>>2);
        pixel = game.display.screenbuffer[i];
        col &= 3; // just in case
        column = w&3;
             if (column==3) pixel = (pixel & B00000011) | (byteColor & B11111100);
        else if (column==2) pixel = (pixel & B00001111) | (byteColor & B11110000);
        else if (column==1) pixel = (pixel & B00111111) | (byteColor & B11000000);
        game.display.screenbuffer[i] = pixel;
    }

}

bool myButton(int x, int y, int w, int h, const char* text){

    bool isClick=false;
    int mx = myMouse.x>>8;
    int my = myMouse.y>>8;
    if(_A[NEW]){
        if(mx >= x && mx <= x+w && my >= y && my <= y+h){isClick=true;}
    }

    // face of button
    drawFillRect2(x,y,x+w,y+h,gui_ltgrey);
    // border
    if(isClick==false){
        myLine(x, y, x+w, y, gui_white);
        myLine(x, y, x, y+h, gui_white);
        myLine(x, y+h, x+w, y+h, gui_black);
        myLine(x+w, y, x+w, y+h, gui_black);
    }else{
        myLine(x, y, x+w, y, gui_black);
        myLine(x, y, x, y+h, gui_black);
        myLine(x, y+h, x+w, y+h, gui_white);
        myLine(x+w, y, x+w, y+h, gui_white);
    }
    //text
    game.display.setInvisibleColor(0);
    print((x+w/2)+isClick,y+2+isClick,text,1);
    game.display.setInvisibleColor(-1);

    return isClick;
}

void drawWindow(int x, int y, int w, int h, const char* winTitle){
    // backdrop
    drawFillRect2(x+1,y+1,x+w,y+h,gui_ltgrey);

    // titlebar
    drawFillRect2(x+1,y+1,x+w,y+8,gui_white);
    myLine(x, y+8, x+w, y+8, gui_black);
    print(x+9,y+2,winTitle);

    myLine(x+1, y, x+w-1, y, gui_black);
    myLine(x+1, y+h, x+w-1, y+h, gui_black);
    myLine(x, y+1, x, y+h-1, gui_black);
    myLine(x+w, y+1, x+w, y+h-1, gui_black);

    // corners
    drawPixel2(x+1, y+1, gui_black);
    drawPixel2(x+w-1, y+1, gui_black);
    drawPixel2(x+1, y+h-1, gui_black);
    drawPixel2(x+w-1, y+h-1, gui_black);

    // button glyphs
    game.display.setInvisibleColor(gui_white);
    drawBitmap(x+w-6,y+2, glyphs[2], 2);
    game.display.setInvisibleColor(-1);

}


void updateMouse(){

    int xd = (((_Left[HELD]-_Right[HELD])*myMouse.analog));
    int yd = (((_Down[HELD]-_Up[HELD])*myMouse.analog));

    // motion gravity etc.
    myMouse.xSpeed += -xd;
    myMouse.ySpeed += yd;
    // add some dampening to speed
    myMouse.xSpeed *= myMouse.decay;
    myMouse.ySpeed *= myMouse.decay;
    myMouse.x += myMouse.xSpeed;
    myMouse.y += myMouse.ySpeed;

    game.display.setInvisibleColor(1);
    drawBitmap(myMouse.x>>8,myMouse.y>>8, gfx_mouse, 2);
    game.display.setInvisibleColor(-1);

}

void updateScreen(){
// Pokitto::setBacklight(float value); // between 0 and 1
    struct tm tm = *localtime(&pokTime);

    for(int t=0; t<max_windows; t++){
        if(window[t].inUse){

            switch(window[t].function){

                case 0: // desktop
                {
                    int p1 = (window[t].y>>2)-1;
                    int p2 = (window[t].y>>2)+1;
                    for(int p=p1; p<p2; p++){
                        //              x, y,        start line, number of lines, bitmap array, bitdepth
                        //drawBitmapLines(0, p ,p ,  2,               seein_angel,  8);
                        drawBitmapLines(0, p ,p ,  2,               leaves,  4);
                    }

                    drawWindow(window[t].x,window[t].y,window[t].w,window[t].h,window[t].title);
                    // time/date icon
                    drawBitmap(window[t].x+8,window[t].y+16,icon_cal,2);
                    game.display.setInvisibleColor(0);
                    print(window[t].x+18,window[t].y+36,"Date/Time",1);
                    game.display.setInvisibleColor(-1);


                    sprintf(tempText, "%s %02d/%02d/%02d %02d:%02d:%02d", wDays[tm.tm_wday],tm.tm_mday, tm.tm_mon+1, (tm.tm_year + 1900)%100,  tm.tm_hour, tm.tm_min, tm.tm_sec);
                    print(window[t].x+window[t].w, window[t].y+2,tempText,2);
                }
                    break;

                case 1: // Time window
                    {
                        int top = window[_backdrop].y;
                        drawWindow(window[t].x,window[t].y+top,window[t].w,window[t].h,window[t].title);

                        myButton(window[t].x+6, top+window[t].y+10, 8*4, 8, wDaysFull[tm.tm_wday]);

                        bool timeChanged = false;

                        sprintf(tempText, "%02d", tm.tm_hour);
                        if( myButton(window[t].x+6, top+window[t].y+20, 8*4, 8, tempText) == true){
                            tm.tm_hour++;
                            timeChanged = true;
                        }
                        sprintf(tempText, "%02d", tm.tm_min);
                        if( myButton(window[t].x+6, top+window[t].y+30, 8*4, 8, tempText) == true){
                            tm.tm_min++;
                            timeChanged = true;
                        }

                        sprintf(tempText, "%02d", tm.tm_mday);
                        if( myButton(window[t].x+6, top+window[t].y+40, 8*4, 8, tempText) == true){
                            tm.tm_mday++;
                            timeChanged = true;
                        }
                        sprintf(tempText, "%02d", tm.tm_mon);
                        if( myButton(window[t].x+6, top+window[t].y+50, 8*4, 8, tempText) == true){
                            tm.tm_mon++;
                            timeChanged = true;
                        }
                        sprintf(tempText, "%02d", tm.tm_year);
                        if( myButton(window[t].x+6, top+window[t].y+60, 8*4, 8, tempText) == true){
                            tm.tm_year++;
                            timeChanged = true;
                        }


                        if(timeChanged == true){
                            toTime = asUnixTime(tm.tm_year + 1900, tm.tm_mon, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec);
                            set_time(toTime);  // 2000, jan, 1st, 00:00:00
                        }
                        //sprintf(tempText, "%s", wDaysFull[tm.tm_wday]); print(window[t].x+6, top+window[t].y+10,tempText);
                        //sprintf(tempText, "<  %02d  >", tm.tm_mday); print(window[t].x+6, top+window[t].y+18,tempText);
                        //sprintf(tempText, "< %s  >", mDays[tm.tm_mon]); print(window[t].x+6, top+window[t].y+24,tempText);
                        //sprintf(tempText, "< %04d >", tm.tm_year + 1900); print(window[t].x+6, top+window[t].y+30,tempText);


                    }
                    break;

                default:
                    drawWindow(window[t].x,window[t].y,window[t].w,window[t].h,window[t].title);
                    break;
            }

        }
    }



    sprintf(tempText,"FPS:%d Bat:%d", (int)Pokitto::core.fps_counter, (unsigned int) (Pokitto::Battery::level));
    print(219,170,tempText,2);

    updateMouse();
}

int addWindow(int x, int y, int w, int h, const char* caption, int function){

    int windowAtFront = 0;
    for(int t=0; t<max_windows; t++){
        window[t].zOrder +=1;
        //if(window[t].atFront == true) windowAtFront = t;
        //window[t].atFront = false;
    }

    bool completed = false;
    int winNum = -1;
    for(int t=0; t<max_windows; t++){
        if(window[t].inUse==false && completed == false){
            window[t].x=x;
            window[t].y=y;
            window[t].w=w;
            window[t].h=h;

            strncpy(window[t].title, caption, 24);
            window[t].title[24] = '\0';

            window[t].function=function;
            window[t].inUse=true;
            window[t].zOrder = 0;
            completed = true;
            winNum = t;
        }
    }

//    drawFillRect2(0,0,220,176,gui_dkgrey);

    return winNum;
}

 int main(){

    startup();

    // update buttons
    myPad = updateButtons(myPad);
    UpdatePad(myPad);
    while(_A[HELD]){
        myPad = updateButtons(myPad);
        UpdatePad(myPad);
    }

    _backdrop = addWindow(-1,-1,221,177,"PokittOS",0);

    int frameNumber = 0;
    while (game.isRunning()) {
        int mx = myMouse.x>>8;
        int my = myMouse.y>>8;

        if(game.update()){
            // update buttons
            myPad = updateButtons(myPad);
            UpdatePad(myPad);

            if(_A[NEW]){

                for(int t=0; t<max_windows; t++){
                    if(window[t].inUse){
                        if(mx>window[t].x && mx<window[t].x+window[t].w && my>window[t].y && my<window[t].y+11){
                            window[t].drag = true;
                            window[t].dragX = mx-window[t].x;
                            window[t].dragY = my-window[t].y;
                            window[t].zOrder = 0;
                        }
                        if(mx>=window[t].x+window[t].w-6 && mx<=window[t].x+window[t].w-2 && my>=window[t].y+2 && my<=window[t].y+7){
                            window[t].inUse=0;
                            //window[t].atFront=0;
                            //window[_backdrop].atFront=1;
                        }
                    }
                }
                // check icons
                if(mx>=8 && mx<=28 && my>=16 && my<36){
                    addWindow(53,33,106,63,"Set Time",1);
                }
            }

            if(_A[RELEASE]){
                for(int t=0; t<max_windows; t++){
                    window[t].drag = false;
                }
            }



            if(_C[NEW]){
                //setMyTime();
                //curTime = time(NULL);
            }

            pokTime = time(NULL);


                for(int t=0; t<max_windows; t++){
                    if(window[t].drag == true){
                        window[t].x = mx-window[t].dragX;
                        window[t].y = my-window[t].dragY;
                        if(window[t].y<-1){
                            window[t].y = -1;
                            myMouse.y = (window[t].y+window[t].dragY)<<8;
                        }
                        if(window[t].y>168){
                            window[t].y = 168;
                            myMouse.y = (window[t].y+window[t].dragY)<<8;
                        }
                        if(window[t].x<-1){
                            window[t].x = -1;
                            myMouse.x = (window[t].x+window[t].dragX)<<8;
                        }
                        if(window[t].x>220-window[t].w){
                            window[t].x = 220-window[t].w;
                            myMouse.x = (window[t].x+window[t].dragX)<<8;
                        }

                        if(window[t].function == 0){
                            for(int y=0; y<88; y++){
                                if(y < (window[t].y+1)/2){
                                    //game.display.scanType[y]=0; // 8bit
                                    game.display.scanType[y]=1; // 4bit
                                }else{
                                    game.display.scanType[y]=2; // 2bit
                                }
                            }
                        }


                    }
                }


            updateScreen();
        }

    }

return 1;
}

